'use strict';

exports.__esModule = true;
exports.inline = inline;
exports.keyframes = keyframes;
exports.fontFace = fontFace;
exports.injectGlobal = injectGlobal;

var _parser = require('./parser');

var _hash = require('./hash');

var _hash2 = _interopRequireDefault(_hash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function extractNameFromProperty(str) {
  var regex = /name\s*:\s*([A-Za-z0-9\-_]+)\s*/gm;
  var match = regex.exec(str);
  if (match) {
    return match[1];
  }
}

function getName(extracted, identifierName, prefix) {
  var parts = [];
  parts.push(prefix);
  if (extracted) {
    parts.push(extracted);
  } else if (identifierName) {
    parts.push(identifierName);
  }
  return parts.join('-');
}

function replaceVarsWithPlaceholders(rules) {
  return rules.map(function (rule) {
    return rule.replace(/var\(--[A-Za-z0-9-_]+-([0-9]+)\)/gm, function (match, p1) {
      return 'xxx' + p1 + 'xxx';
    });
  });
}

function replaceApplyWithPlaceholders(rules) {
  return rules.map(function (rule) {
    return rule.replace(/@apply\s+--[A-Za-z0-9-_]+-([0-9]+)/gm, function (match, p1) {
      return 'xxx' + p1 + 'xxx';
    });
  });
}

function inline(quasi, identifierName, prefix, inlineVars) {
  var strs = quasi.quasis.map(function (x) {
    return x.value.cooked;
  });
  var hash = (0, _hash2.default)([].concat(strs) // todo - add current filename?
  );var name = getName(extractNameFromProperty(strs.join('xxx')), identifierName, prefix);
  var hasApply = void 0;
  var src = strs.reduce(function (arr, str, i) {
    arr.push(str);
    if (i !== quasi.expressions.length) {
      // todo - test for preceding @apply
      var applyMatch = /@apply\s*$/gm.exec(str);
      if (applyMatch) {
        hasApply = true;
        arr.push('--' + name + '-' + hash + '-' + i);
      } else arr.push('var(--' + name + '-' + hash + '-' + i + ')');
    }
    return arr;
  }, []).join('').trim();

  var rules = (0, _parser.parseCSS)('.' + name + '-' + hash + ' { ' + src + ' }');
  if (inlineVars || hasApply) {
    rules = replaceVarsWithPlaceholders(rules);
  }
  if (hasApply) {
    rules = replaceApplyWithPlaceholders(rules);
  }
  return { hash: hash, name: name, rules: rules, isStatic: !hasApply && !inlineVars };
}

function keyframes(quasi, identifierName, prefix, inlineVars) {
  var strs = quasi.quasis.map(function (x) {
    return x.value.cooked;
  });
  var hash = (0, _hash2.default)([].concat(strs));
  var name = getName(extractNameFromProperty(strs.join('xxx')), identifierName, prefix);
  var hasApplyOrVar = false;
  var src = strs.reduce(function (arr, str, i) {
    arr.push(str);
    if (i !== quasi.expressions.length) {
      hasApplyOrVar = true;
      // todo - test for preceding @apply
      var applyMatch = /@apply\s*$/gm.exec(str);
      if (applyMatch) {
        arr.push('--name-hash-' + i);
      } else arr.push('var(--name-hash-' + i + ')');
    }
    return arr;
  }, []).join('').trim();
  var rules = (0, _parser.parseCSS)('{ ' + src + ' }', { nested: false });
  if (hasApplyOrVar) {
    rules = replaceVarsWithPlaceholders(rules);
    rules = replaceApplyWithPlaceholders(rules);
  }
  return { hash: hash, name: name, rules: rules, isStatic: !hasApplyOrVar && !inlineVars };
}

function fontFace(quasi, inlineVars) {
  var strs = quasi.quasis.map(function (x) {
    return x.value.cooked;
  });
  var hasVar = void 0;
  var src = strs.reduce(function (arr, str, i) {
    arr.push(str);
    if (i !== quasi.expressions.length) {
      hasVar = true;
      arr.push('var(--name-hash-' + i + ')');
    }
    return arr;
  }, []).join('').trim();
  var rules = (0, _parser.parseCSS)('@font-face {' + src + '}');
  if (hasVar) {
    rules = replaceVarsWithPlaceholders(rules);
  }
  return { rules: rules, isStatic: !inlineVars && !hasVar };
}

function injectGlobal(quasi, inlineVars) {
  var strs = quasi.quasis.map(function (x) {
    return x.value.cooked;
  });
  var hasApplyOrVar = false;
  var src = strs.reduce(function (arr, str, i) {
    arr.push(str);
    if (i !== quasi.expressions.length) {
      hasApplyOrVar = true;
      // todo - test for preceding @apply
      var applyMatch = /@apply\s*$/gm.exec(str);
      if (applyMatch) {
        arr.push('--name-hash-' + i);
      } else arr.push('var(--name-hash-' + i + ')');
    }
    return arr;
  }, []).join('').trim();
  var rules = (0, _parser.parseCSS)(src);
  if (hasApplyOrVar) {
    rules = replaceVarsWithPlaceholders(rules);
    rules = replaceApplyWithPlaceholders(rules);
  }
  return { rules: rules, isStatic: !inlineVars && !hasApplyOrVar };
}